import * as sqlite from 'sqlite3';
import sql, { SQLQuery } from '@databases/sql';
export { sql };
export declare enum DatabaseConnectionMode {
    ReadOnly,
    ReadWrite,
    ReadWriteCreate,
    ReadCreate,
    Create
}
export interface DatabaseConnectionOptions {
    /**
     * Sets the busy timeout. Must be a postive integer if provided.
     *
     * @see https://www.sqlite.org/c3ref/busy_timeout.html
     */
    busyTimeout?: number;
    /**
     * Defaults to DatabaseConnectionMode.ReadWriteCreate
     */
    mode?: DatabaseConnectionMode;
    /**
     * Enable long stack traces for debugging. This is global
     * and cannot be disabled once enabled.
     */
    verbose?: boolean;
}
export declare class DatabaseTransaction {
    private readonly _database;
    constructor(database: sqlite.Database);
    query(query: SQLQuery): Promise<any[]>;
    stream(query: SQLQuery): AsyncIterableIterator<any>;
}
export declare const IN_MEMORY = ":memory:";
export declare class DatabaseConnection {
    private readonly _database;
    private readonly _mutex;
    constructor(filename: string, options?: DatabaseConnectionOptions);
    query(query: SQLQuery): Promise<any[]>;
    stream(query: SQLQuery): AsyncIterableIterator<any>;
    tx<T>(fn: (db: DatabaseTransaction) => Promise<T>): Promise<T>;
    dispose(): Promise<void>;
}
export default function connect(filename?: string, options?: DatabaseConnectionOptions): DatabaseConnection;
